# -*- coding: utf-8 -*-


from django.core.management import BaseCommand
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta

import subprocess
import string
import random
import json
import os
from PyPDF2 import PdfFileWriter, PdfFileReader

from aparnik.utils.utils import round
from aparnik.contrib.filefields.models import FileField
from aparnik.packages.educations.courses.models import CourseFile


def randomPasswordAndIV():
    """Generate a random password """
    password = ''
    iv = ''

    for i in range(64):
        password += random.choice(string.hexdigits)
    for i in range(32):
        iv += random.choice(string.hexdigits)
    return password.lower(), iv.lower()


class Command(BaseCommand):
    # Show this when the user types help
    help = "Encrypt File fields"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        queryset = FileField.objects.filter(is_lock=True)

        for file in queryset:
            # check file exist
            # ممکن ه که هنوز فرآیند آپلود به اتمام نرسیده باشد
            if not file.type:
                continue
            path = file.file_direct.path
            infile = path
            outfile = '%s.enc' % path

            if file.is_decrypt_needed:
                # decrypt
                # pdf
                # movies
                if file.type == FileField.FILE_PDF:
                    # with open(infile, "rb") as ifile, open(outfile, "wb") as ofile:
                    #     reader = PdfFileReader(ifile)
                    #     print(str(file.password))
                    #     if reader.isEncrypted:
                    #         reader.decrypt(unicode('1da72c67efb2ccdcf43fef8c6afacdd405659effaadcdae31cabb2ead8df43ab'))
                    #
                    #     writer = PdfFileWriter()
                    #     for i in range(reader.getNumPages()):
                    #         writer.addPage(reader.getPage(i))
                    #     writer.write(ofile)
                    # TODO: fix pdf decrypt
                    continue
                else:

                    subprocess.call(
                        'openssl enc -aes-256-cbc -d -nosalt -K %s -iv %s -in %s -out %s' % (
                        file.password, file.iv, infile, outfile),
                        shell=True, close_fds=True)

                os.remove(path)
                os.rename(outfile, infile)
                file.is_decrypt_needed = False
                file.password = None
                file.iv = None
            else:
               # find type
                if not file.type:
                    file.find_type()
                    if not file.type:
                        continue

                if file.type == FileField.FILE_VOICE or file.type == FileField.FILE_MOVIE:
                    ffprobe_properties = "ffprobe -i '%s' -v quiet -print_format json -show_format -hide_banner" % path
                    metadata = subprocess.check_output(
                        ffprobe_properties,
                        shell=True, close_fds=True)
                    metadata = json.loads(metadata)

                    if 'duration' in metadata['format']:
                        duration = metadata['format']['duration']
                        file.seconds = int(round(duration))
                        for f in file.shop_file_obj.all():

                            if isinstance(f, CourseFile):
                                f.seconds = file.seconds
                                f.save()
                    else:
                        continue

                # Encrypt
                if file.is_encrypt_needed:
                    # Generate Key and IV
                    key, iv = randomPasswordAndIV()

                    if file.type == FileField.FILE_PDF:

                        iv = None
                        # Encrypt PDF

                        with open(infile, "rb") as ifile, open(outfile, "wb") as ofile:
                            reader = PdfFileReader(ifile)
                            writer = PdfFileWriter()
                            for i in range(reader.getNumPages()):
                                writer.addPage(reader.getPage(i))
                            writer.encrypt(str(key))
                            writer.write(ofile)

                    else:

                        # Encrypt Etc
                        subprocess.call('openssl enc -aes-256-cbc -nosalt -K %s -iv %s -in %s -out %s'%(key, iv, infile, outfile), shell=True, close_fds=True)

                    os.remove(path)
                    os.rename(outfile, infile)
                    file.password = key
                    file.iv = iv

            # Finish and release lock
            file.is_lock = False
            file.save()

        finished_time = now()

        print(('encrypt done %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('basemodels', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sliders', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Dossier',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('activity', models.TextField(verbose_name='activity')),
                ('start_date', models.DateTimeField(verbose_name='Start Date')),
                ('finish_date', models.DateTimeField(verbose_name='Finish Date')),
            ],
            options={
                'verbose_name': 'Dossier',
                'verbose_name_plural': 'Dossiers',
            },
            bases=('basemodels.basemodel',),
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('base_objects', django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name='Teacher',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('biography', models.TextField(verbose_name='Biography')),
                ('start_date', models.DateTimeField(blank=True, null=True, verbose_name='Start Date')),
                ('slider_obj', models.ManyToManyField(to='sliders.Slider', verbose_name='Slider')),
                ('user_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Teacher',
                'verbose_name_plural': 'Teachers',
            },
            bases=('basemodels.basemodel',),
        ),
        migrations.AddField(
            model_name='dossier',
            name='teacher_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='teachers.Teacher', verbose_name='teacher'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


import aparnik.utils.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('basemodels', '0001_initial'),
        ('socials', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Support',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('online', models.CharField(blank=True, max_length=255, null=True, verbose_name='Online')),
                ('offline', models.CharField(blank=True, max_length=255, null=True, verbose_name='Offline')),
                ('phone', aparnik.utils.fields.PhoneField(blank=True, max_length=11, null=True, validators=[django.core.validators.RegexValidator(code=b'nomatch', message='phone is not valid, please insert with code', regex=b'^0(?!0)\\d{2}([0-9]{8})$')], verbose_name='Phone')),
                ('online_days', models.CharField(blank=True, max_length=255, null=True, verbose_name='Online Days')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('socials', models.ManyToManyField(to='socials.SocialNetwork', verbose_name='Socials')),
            ],
            options={
                'verbose_name': 'Support',
                'verbose_name_plural': 'Supports',
            },
            bases=('basemodels.basemodel',),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-03 00:41


from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('courses', '0005_remove_course_is_free'),
        ('progresses', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProgressSummary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('percentage', models.IntegerField(verbose_name='Percentage')),
                ('model', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='progress_summaries', to='courses.Course', verbose_name='Models')),
                ('user_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='progress_user', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Progress Summary',
                'verbose_name_plural': 'Progress Summary',
            },
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.conf import settings
from jalali_date import datetime2jalali

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.basemodels.admin import BaseModelAdmin
from .models import Category


# Register your models here.
def get_created_at(obj):
    return datetime2jalali(obj.created_at).strftime(settings.JALALI_FORMAT)


#
# class CategoryAdmin(admin.ModelAdmin):
#     # fields = ['title', 'description', 'image', 'parent_obj']
#     list_display = ['title', 'parent_obj', get_created_at]
#
#     # search_fields = ['email', 'article', 'created_at']
#     # list_filter = ['email', 'article', 'created_at']
#     # readonly_fields = []
#     # actions = []
#
#     class Meta:
#         Category
class CategoryAdmin(BaseModelAdmin):
    fields = ['title', 'description', 'image', 'parent_obj']
    list_display = ['title', 'parent_obj', get_created_at]
    search_fields = ['title', 'parent_obj__title']
    list_filter = ['parent_obj', 'created_at']
    raw_id_fields = ['image', 'parent_obj']
    exclude = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = CategoryAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Category
    # search_fields = ['email', 'article']
    # list_filter = ['email', 'article', 'created_at']
    # readonly_fields = []
    # actions = []


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.admin import SegmentAdmin
    from .models import CategorySegment

    class CategorySegmentAdmin(SegmentAdmin):
        fields = []
        list_display = []
        search_fields = []
        list_filter = []
        raw_id_fields = []
        exclude = []
        dynamic_raw_id_fields = []

        def __init__(self, *args, **kwargs):
            Klass = CategorySegmentAdmin
            Klass_parent = SegmentAdmin

            super(Klass, self).__init__(*args, **kwargs)
            self.fields = Klass_parent.fields + self.fields
            self.list_display = Klass_parent.list_display + self.list_display
            self.list_filter = Klass_parent.list_filter + self.list_filter
            self.search_fields = Klass_parent.search_fields + self.search_fields
            self.exclude = Klass_parent.exclude + self.exclude
            self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
            self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

        class Meta:
            model = CategorySegment


    admin.site.register(CategorySegment, CategorySegmentAdmin)

admin.site.register(Category, CategoryAdmin)
# admin.site.register(CategoryCourse, CategoryCourseAdmin)
# admin.site.register(CategoryLibrary, CategoryLibraryAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-08 11:32


from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('files', '0004_file_cover'),
    ]

    operations = [
        migrations.AddField(
            model_name='file',
            name='publish_date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='\\u062a\\u0627\\u0631\\u06cc\\u062e \\u0627\\u0646\\u062a\\u0634\\u0627\\u0631'),
        ),
    ]



from rest_framework import serializers

from aparnik.contrib.audits.models import Audit
from aparnik.contrib.basemodels.api.admin.serializers import BaseModelAdminSerializer


class AuditAdminListSerializer(BaseModelAdminSerializer):

    def __init__(self, *args, **kwargs):
        super(AuditAdminListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Audit
        fields = BaseModelAdminSerializer.Meta.fields + [
            'ip',
        ]

        read_only_fields = BaseModelAdminSerializer.Meta.read_only_fields + [
            'ip',
        ]



# -*- coding: utf-8 -*-


from django.db.models import Sum, Avg
from django.core.management import BaseCommand
from django.contrib.auth import get_user_model
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta

from aparnik.packages.educations.courses.models import Course, CourseSummary
from aparnik.packages.educations.progresses.models import Progresses, ProgressSummary
from aparnik.contrib.basemodels.models import BaseModel
import datetime

User = get_user_model()


class Command(BaseCommand):
    # Show this when the user types help
    help = "calculate for Progress Model"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        #
        reindex = False
        if Progresses.objects.update_needed().count() > 0:
            reindex = True

        if reindex:
            self.reindex()

        Progresses.objects.update_needed().update(update_needed=False)
        finished_time = now()

        print(('reindex progress %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))

    def reindex(self):
        models = []
        for user_id in Progresses.objects.update_needed().values_list('user_obj', flat=True).distinct():
            user = User.objects.get(pk=user_id)
            # prefetch_related('file_obj__course_obj').
            progresses_user = Progresses.objects.this_user(user=user).prefetch_related('file_obj__course_obj__parent_obj')
            courses_ids = progresses_user.values_list('file_obj__course_obj', flat=True).distinct()
            parents = []
            # CourseSummary.objects.filter(course__id__in=courses_ids).annotate(total_pages=Sum('book__pages'))
            # فقط دوره هایی که فایل دارند حساب می شوند و رکورد آن ها درج می شود.
            for course_id in courses_ids:
                course = Course.objects.get(pk=course_id)
                if course.parent_obj:
                    parents.append(course.parent_obj)

                total_time_seconds = CourseSummary.objects.filter(course=course).aggregate(total_time_seconds=Sum('total_time_seconds'))['total_time_seconds'] or 0.0
                total_complete = progresses_user.filter(file_obj__course_obj=course).aggregate(total_time_seconds=Sum('file_obj__seconds'))['total_time_seconds'] or 0.0
                try:
                    percentage = (total_complete / total_time_seconds) * 100
                except:
                    percentage = 0.0

                try:

                    progress_summary = ProgressSummary.objects.get(user_obj=user, model=course)
                    progress_summary.percentage = percentage
                    progress_summary.save()
                except Exception as e:

                    ProgressSummary.objects.create(user_obj=user, model=course, percentage=percentage)

            # حساب درصد دوره های بالا دستی از روی سامری!
            while(parents):
                parents_tmp = list(set(parents))
                parents = []
                for obj in parents_tmp:
                    if obj.parent_obj:
                        parents.append(obj.parent_obj)
                    percentage = ProgressSummary.objects.active().filter(user_obj=user, model__parent_obj=obj).aggregate(percentage_avg=Avg('percentage'))['percentage_avg'] or 0.0
                    try:

                        progress_summary = ProgressSummary.objects.get(user_obj=user, model=obj)
                        progress_summary.percentage = percentage
                        progress_summary.save()
                    except Exception as e:

                        ProgressSummary.objects.create(user_obj=user, model=obj, percentage=percentage)



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.contrib.users.admin import get_user_search_fields
from .models import Audit


class AuditAdmin(BaseModelAdmin):
    fields = ['action', 'ip', 'user_obj', 'extra']
    list_display = ['action', 'ip', 'user_obj', 'created_at']
    search_fields = get_user_search_fields('user_obj') + ['ip', 'user_obj', 'extra', ]
    list_filter = ['action']
    raw_id_fields = ['user_obj']
    exclude = []
    dynamic_raw_id_fields = []        # exclude = ('password', 'iv',)
    inlines = []

    def __init__(self, *args, **kwargs):
        Klass = AuditAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields
        self.inlines = Klass_parent.inlines + self.inlines

    class Meta:
        model = Audit


admin.site.register(Audit, AuditAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-11 13:51


from django.db import migrations, models
import tagulous.models.fields
import tagulous.models.models


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0005_basemodel_is_show_only_for_super_user'),
    ]

    operations = [
        migrations.CreateModel(
            name='Tagulous_BaseModel_tags',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('slug', models.SlugField()),
                ('count', models.IntegerField(default=0, help_text='Internal counter of how many times this tag is in use')),
                ('protected', models.BooleanField(default=False, help_text='Will not be deleted when the count reaches 0')),
            ],
            options={
                'ordering': ('name',),
                'abstract': False,
            },
            bases=(tagulous.models.models.BaseTagModel, models.Model),
        ),
        migrations.AlterUniqueTogether(
            name='tagulous_basemodel_tags',
            unique_together=set([('slug',)]),
        ),
        migrations.AddField(
            model_name='basemodel',
            name='tags',
            field=tagulous.models.fields.TagField(_set_tag_meta=True, help_text='Enter a comma-separated tag string', to='basemodels.Tagulous_BaseModel_tags'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-25 11:09


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0002_auto_20181125_1057'),
    ]

    operations = [
        migrations.AlterField(
            model_name='setting',
            name='value',
            field=models.TextField(verbose_name='Value'),
        ),
    ]



from django.conf.urls import url, include
from .views import FileListAPIView, FileDownloadRequestAPIView

app_name = 'files'

urlpatterns = [
    url(r'^$', FileListAPIView.as_view(), name='list'),
    # url(r'^(?P<file_id>\d+)/$', FileDetailAPIView.as_view(), name='details'),
    # url(r'^(?P<course_id>\d+)/$', BaseFileFileListAPIView.as_view(), name='list-file'),
    # url(r'^$', BaseFileLibraryListAPIView.as_view(), name='list-library'),

    url(r'^(?P<file_id>\d+)/download-request/$', FileDownloadRequestAPIView.as_view(), name='download-request'),
]



from django.utils.translation import ugettext_lazy as _
from django.shortcuts import get_object_or_404, get_list_or_404
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.permissions import ( AllowAny, IsAuthenticated )
from rest_framework.views import APIView
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError

from ..models import Coupon
from aparnik.packages.shops.orders.models import Order
from aparnik.packages.shops.orders.api.serializers import OrderSummarySerializer


class CouponAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user

        content = {
            'success': False,
            'msg': '',
            'data': {
            },
        }
        coupon_code = self.request.data.get('coupon_code', None)
        order = get_object_or_404(Order.objects.get_this_user(user=user), id=id)

        try:

            if not coupon_code:
                raise ValidationError({'coupon_code': [_('This field is required.')]})

            # payment = Payment.objects.request_pay(order=order, method=method, call_back_url=call_back_url, user=user)
            coupon = Coupon.objects.add_coupon(code=coupon_code, user=user, order=order)
            order.coupon = coupon
            order.save()
            content['data']['order'] = OrderSummarySerializer(order, many=False, read_only=True, context={'request': self.request}).data
            content['success'] = True
            status = HTTP_200_OK
            return Response(content, status=status)
        except Exception as e:
            raise ValidationError(as_serializer_error(e))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('products', '0001_initial'),
        ('files', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaseCourse',
            fields=[
                ('product_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='products.Product')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
            ],
            options={
                'verbose_name': 'Base Course',
                'verbose_name_plural': 'Bases Courses',
            },
            bases=('products.product',),
        ),
        migrations.CreateModel(
            name='CourseFile',
            fields=[
                ('file_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='files.File')),
                ('time', models.TimeField(verbose_name='\\u0632\\u0645\\u0627\\u0646')),
                ('sort', models.IntegerField(default=0, verbose_name='Sort')),
            ],
            options={
                'verbose_name': '\\u067e\\u0631\\u0648\\u0646\\u062f\\u0647',
                'verbose_name_plural': 'Files',
            },
            bases=('files.file',),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-03 22:33


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0003_basemodel_update_needed'),
    ]

    operations = [
        migrations.AlterField(
            model_name='basemodel',
            name='update_needed',
            field=models.BooleanField(default=False, verbose_name='\\u0646\\u06cc\\u0627\\u0632 \\u0628\\u0647 \\u0628\\u0631\\u0648\\u0632\\u0631\\u0633\\u0627\\u0646\\u06cc'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 18:26


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0003_auto_20181026_1745'),
    ]

    operations = [
        migrations.AddField(
            model_name='filefield',
            name='title',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='\\u0639\\u0646\\u0648\\u0627\\u0646'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-13 17:13


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('addresses', '0001_initial'),
        ('orders', '0002_auto_20181026_1301'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='address_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='addresses.UserAddress', verbose_name='Order Address'),
        ),
    ]



from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model
from django.utils.timezone import now
from django.contrib.auth.models import Permission
from aparnik.settings import aparnik_settings
from aparnik.contrib.users.models import DeviceLogin
from aparnik.utils.utils import convert_iran_phone_number_to_world_number
import logging

User = get_user_model()


class AuthBackend(ModelBackend):

    def authenticate(self, request, username=None, password=None, **kwargs):
        if username is None:

            username = kwargs.get(User.USERNAME_FIELD)
        try:
            username = convert_iran_phone_number_to_world_number(username)
            user = User._default_manager.get_by_natural_key(username)
        except User.DoesNotExist:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (#20760).
            User().set_password(password)
        else:
            # return user
            is_correct_password = False
            if aparnik_settings.USER_LOGIN_WITH_PASSWORD:
                # SMS
                is_correct_password = user.passwd == password
            else:
                is_correct_password = user.OTAVerify(password)

            if user.is_can_login() and is_correct_password:
                user.last_login = now()
                user.save()
                # because we can access the request in jwt :| we use device login to find last login
                # device_id = request.data.get('device_id', None)
                # if device_id is not None:

                return user
        return None

    def get_user(self, user_id):
        try:
            user = User.objects.get(sys_id=user_id)
            if user.is_active:
                return user
            return None
        except User.DoesNotExist:
            logging.getLogger("error_logger").error("user with %(user_id)d not found")
            return None


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-05 20:22


from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='FieldList',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('model', models.CharField(max_length=100, verbose_name='model')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('is_enable', models.BooleanField(default=True, verbose_name='Is enable')),
                ('is_sharable', models.BooleanField(verbose_name='Is sharable')),
                ('default', models.CharField(blank=True, max_length=200, null=True, verbose_name='Default')),
            ],
            options={
                'verbose_name': 'Field',
                'verbose_name_plural': 'Fields',
            },
        ),
        migrations.CreateModel(
            name='Management',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('application', models.CharField(max_length=100, verbose_name='Application')),
                ('constraint', models.CharField(max_length=200, verbose_name='constraints')),
                ('start_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Start date')),
                ('end_date', models.DateTimeField(blank=True, null=True, verbose_name='End date')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('fields', models.ManyToManyField(to='managements.FieldList', verbose_name='fields')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group', verbose_name='group')),
                ('parent_obj', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='managements.Management', verbose_name='Parent')),
            ],
            options={
                'verbose_name': 'Admin Permission',
                'verbose_name_plural': 'Admin Permissions',
            },
        ),
        migrations.CreateModel(
            name='ManagementActions',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=50, verbose_name='Function title')),
                ('description', models.CharField(max_length=100, verbose_name='Description')),
            ],
            options={
                'verbose_name': 'Admin action',
                'verbose_name_plural': 'Admin actions',
            },
        ),
        migrations.CreateModel(
            name='ManagementPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=50, verbose_name='Title')),
                ('description', models.CharField(max_length=100, verbose_name='Description')),
            ],
        ),
        migrations.CreateModel(
            name='ManagementRoles',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=30, verbose_name='Role name')),
                ('management_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='managements.Management', verbose_name='Management group')),
            ],
            options={
                'verbose_name': 'Role',
                'verbose_name_plural': 'Roles',
            },
        ),
        migrations.AddField(
            model_name='fieldlist',
            name='actions',
            field=models.ManyToManyField(to='managements.ManagementActions', verbose_name='Actions'),
        ),
        migrations.AddField(
            model_name='fieldlist',
            name='permission',
            field=models.ManyToManyField(to='managements.ManagementPermission', verbose_name='Management Permission'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib.auth import get_user_model
from django.db import models
from django.db.models import Q, Count, Sum, F
from django.db.models.functions import Coalesce

from django.utils.translation import ugettext_lazy as _

from aparnik.utils.utils import field_with_prefix
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.packages.educations.courses.models import Course, CourseFile

User = get_user_model()


# Create your models here.
class ProgressSummaryManager(models.Manager):

    def get_queryset(self):
        return super(ProgressSummaryManager, self).get_queryset()

    def active(self):
        return self.get_queryset()

    def this_user(self, user):
        if not user.is_authenticated:
            return ProgressSummary.objects.none()
        return self.active().filter(user_obj=user)


class ProgressSummary(models.Model):
    percentage = models.FloatField(verbose_name=_('Percentage'))
    model = models.ForeignKey(Course, related_name='progress_summaries', on_delete=models.CASCADE, verbose_name=_('Models'))
    user_obj = models.ForeignKey(User, related_name='progress_user', on_delete=models.CASCADE, verbose_name=_('User'))

    objects = ProgressSummaryManager()

    class Meta:
        verbose_name = _('Progress Summary')
        verbose_name_plural = _('Progress Summary')

    def __str__(self):
        return '%s' % self.id

    @staticmethod
    def sort_progress(return_key='progress', prefix=''):
        sort = {
            return_key: {
                'label': 'پراگرس',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Coalesce(Sum(field_with_prefix('percentage', prefix)), 0)
                },
                'key_sort': 'sort_count',
            }
        }
        return sort


    @staticmethod
    def sort_user_participation(return_key='users_progress', prefix=''):
        sort = {
            return_key: {
                'label': 'شرکت کاربر',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count(field_with_prefix('user_obj', prefix='course__progress_summaries'))
                },
                'key_sort': 'sort_count',
            }
        }
        return sort


class ProgressesManager(BaseModelManager):

    def get_queryset(self):
        return super(ProgressesManager, self).get_queryset()

    def active(self):
        return super(ProgressesManager, self).active().filter(is_published=True)

    def this_user(self, user, file_obj=None):
        if not user.is_authenticated:
            return ProgressSummary.objects.none()

        queryset = self.active().filter(user_obj=user)
        if file_obj:
            queryset = queryset.filter(file_obj=file_obj)

        return queryset


class Progresses(BaseModel):
    STATUS_NOTSTARTED = 'SN'
    STATUS_STARTED = 'SS'
    STATUS_MANUALCOMPLETE = 'SM'
    STATUS_AUTOCOMPLETE = 'SA'
    STATUS = (
        (STATUS_NOTSTARTED, _('Not started')),
        (STATUS_STARTED, _('Started')),
        (STATUS_MANUALCOMPLETE, _('Manual complete')),
        (STATUS_AUTOCOMPLETE, _('Auto complete')),
    )
    is_published = models.BooleanField(default=True, verbose_name=_('Published'))
    user_obj = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('User'))
    file_obj = models.ForeignKey(CourseFile, on_delete=models.CASCADE, verbose_name=_('File'))
    status = models.CharField(max_length=2, choices=STATUS, default=STATUS_NOTSTARTED, verbose_name=_(
        'Status'))  # Determines if the file not started, started, completed automatically or completed manually by user

    objects = ProgressesManager()

    def __init__(self, *args, **kwargs):
        super(Progresses, self).__init__(*args, **kwargs)

    class Meta:
        verbose_name = _('Progress')
        verbose_name_plural = _('Progresses')

    def __str__(self):
        return str(self.status)



from django.urls import reverse
from django.http import HttpRequest
from django.contrib.auth.models import AnonymousUser

import decimal
from math import log, floor

from aparnik.settings import aparnik_settings
from aparnik.contrib.settings.models import Setting


def document_directory_path(instance, filename, folder='file'):
    from django.utils.timezone import now
    import random
    return '{0}/{1}-{2}.{3}.{4}'.format(folder, filename.split(".")[0], now(), random.randint(0, 1000000),
                                        filename.split(".")[-1])


def convert_iran_phone_number_to_world_number(phone):
    if len(phone) == 10:

        phone = '+98' + phone
    elif len(phone) == 11:

        phone = '+98' + phone[1:]

    if phone[0:1] == '+':
        phone = '00' + phone[1:]

    if phone[0:4] == '0098' and phone[4] == 0:
        phone = phone[0:4] + phone[5:]

    return phone


def is_app_installed(app):
    from django.apps import apps
    return apps.is_installed(app)


def append_absolute_uri(request, urls):
    if request:
        urls_build = {}
        for key, ur in list(urls.items()):
            urls_build[key] = request.build_absolute_uri(ur)
    else:
        urls_build = urls

    return urls_build


def get_url_install(request=None):
    url = {
    }
    # User
    url['login'] = reverse('aparnik-api:users:login')
    url['user-register-with-password'] = reverse('aparnik-api:users:create')
    url['user-request-send-sms'] = reverse('aparnik-api:users:send-sms')
    url['user-verify-sms'] = reverse('aparnik-api:users:verify-sms')
    url['user-forget-password'] = reverse('aparnik-api:users:forget-password')
    url['logout'] = reverse('aparnik-api:users:logout')
    url['FCM-add-token'] = reverse('aparnik-api:users:FCM-add-token')
    url['verify'] = reverse('aparnik-api:users:verify')
    url['token'] = reverse('aparnik-api:users:token')
    url['user-list'] = reverse('aparnik-api:users:list')
    url['user-subset-list'] = reverse('aparnik-api:users:subset')

    # addresses
    url['user-addresses-list'] = reverse('aparnik-api:addresses:list')
    url['user-addresses-create'] = reverse('aparnik-api:addresses:create')

    # bank accounts
    url['user-bankaccounts-list'] = reverse('aparnik-api:bankaccounts:list')
    url['user-banknames-list'] = reverse('aparnik-api:bankaccounts:bankname-list')
    url['user-bankaccounts-create'] = reverse('aparnik-api:bankaccounts:create')

    url['invitations-list'] = reverse('aparnik-api:invitations:list')
    url['about-us'] = reverse('aparnik-api:aboutus:detail')

    # files
    url['s3-sign'] = reverse('aparnik-api:files:s3-sign')
    url['file-create'] = reverse('aparnik-api:files:create')

    # Province
    url['province-list'] = reverse('aparnik-api:provinces:list')

    # Notification
    url['notification-list'] = reverse('aparnik-api:notifications:list')
    url['notification-reads-all'] = reverse('aparnik-api:notifications:reads-all')

    # Notify me
    url['notifies-me-list'] = reverse('aparnik-api:notifiesme:list')

    # supports
    url['supports-list'] = reverse('aparnik-api:supports:list')

    # FAQ
    url['faq-detail'] = reverse('aparnik-api:faq:detail')

    # termsandconditions
    url['termsandconditions-detail'] = reverse('aparnik-api:termsandconditions:detail')

    # education degree
    url['educations-degree-list'] = reverse('aparnik-api:educations:educations:degree-list')
    url['educations-field-subject-list'] = reverse('aparnik-api:educations:educations:field-subject-list')
    url['educations-institude-list'] = reverse('aparnik-api:educations:educations:institude-list')

    # order
    url['order-charge-wallet'] = reverse('aparnik-api:shops:orders:charge-wallet')
    url['order-list'] = reverse('aparnik-api:shops:orders:list')
    url['order-add'] = reverse('aparnik-api:shops:orders:add')

    # Product
    url['products-sort'] = reverse('aparnik-api:shops:products:sort')
    url['products-list'] = reverse('aparnik-api:shops:products:list')

    # Products sharing
    url['productssharing-list'] = reverse('aparnik-api:shops:productssharing:list')

    # payments
    url['payments-list'] = reverse('aparnik-api:shops:payments:list')

    # subscriptions
    url['subscriptions-list'] = reverse('aparnik-api:shops:subscriptions:list')

    # vouchers
    url['vouchers-list'] = reverse('aparnik-api:shops:vouchers:list')

    # course
    url['course-list'] = reverse('aparnik-api:educations:courses:list')
    url['course-user-list'] = reverse('aparnik-api:educations:courses:user')
    url['course-create'] = reverse('aparnik-api:educations:courses:create')

    # book
    url['book-list'] = reverse('aparnik-api:educations:books:list')
    url['book-user-list'] = reverse('aparnik-api:educations:books:user')

    # Bookmark
    url['bookmark-user-list'] = reverse('aparnik-api:bookmarks:list')

    # Review
    url['review-user-list'] = reverse('aparnik-api:reviews:list')

    # QA
    url['qa-user-list'] = reverse('aparnik-api:qa:list')
    url['qa-sort'] = reverse('aparnik-api:qa:sort')

    # Contact us
    url['contact-us-list'] = reverse('aparnik-api:contactus:list')
    url['contact-us-create'] = reverse('aparnik-api:contactus:create')

    # news
    url['news-list'] = reverse('aparnik-api:news:list')

    # CoSale
    url['co-sale-users-list'] = reverse('aparnik-api:shops:cosales:user-list')
    url['co-sale-list'] = reverse('aparnik-api:shops:cosales:list')

    urls = append_absolute_uri(request, url)

    # admin area
    urls_admin = {

    }

    if request and request.user.is_authenticated and request.user.is_admin:

        urls_admin['filefields-list'] = reverse('aparnik-api:files:list')

        urls_admin['audits-sort'] = reverse('aparnik-api:audits:admin:sort')
        urls_admin['audits-list'] = reverse('aparnik-api:audits:admin:list')

        urls_admin['users-sort'] = reverse('aparnik-api:users:admin:sort')
        urls_admin['users-list'] = reverse('aparnik-api:users:admin:list')

        urls_admin['products-sort'] = reverse('aparnik-api:shops:products:admin:sort')
        urls_admin['products-list'] = reverse('aparnik-api:shops:products:admin:list')

        urls_admin['orders-sort'] = reverse('aparnik-api:shops:orders:admin:sort')
        urls_admin['orders-list'] = reverse('aparnik-api:shops:orders:admin:list')

        urls_admin['coupons-sort'] = reverse('aparnik-api:shops:coupons:admin:sort')
        urls_admin['coupons-list'] = reverse('aparnik-api:shops:coupons:admin:list')

        urls_admin['courses-sort'] = reverse('aparnik-api:educations:courses:admin:sort')
        urls_admin['courses-list'] = reverse('aparnik-api:educations:courses:admin:list')

        urls_admin['teachers-sort'] = reverse('aparnik-api:educations:teachers:admin:sort')
        urls_admin['teachers-list'] = reverse('aparnik-api:educations:teachers:admin:list')



    urls_admin = append_absolute_uri(request, urls_admin)

    properties = {}
    for setting in Setting.objects.home_properties():
        properties[setting.key] = setting.get_value()

    return urls, {
        'properties': properties,
        'AWS_ACTIVE': aparnik_settings.AWS_ACTIVE,
        'USER_LOGIN_WITH_PASSWORD': aparnik_settings.USER_LOGIN_WITH_PASSWORD
    }, urls_admin


def get_pages(request):
    from aparnik.contrib.pages.models import Page
    from aparnik.contrib.pages.api.serializers import PageListPolymorphicSerializer
    pages = PageListPolymorphicSerializer(Page.objects.home(), many=True, read_only=True,
                                          context={'request': request}).data

    return pages


def get_request():
    request = HttpRequest()
    server_url = 'aparnik.com'
    server_port = '80'
    try:
        setting = Setting.objects.get(key='SERVER_NAME')
        server_url = setting.get_value()
    except:
        pass

    try:
        setting = Setting.objects.get(key='SERVER_PORT')
        server_port = setting.get_value()
    except:
        pass

    if server_port == '443':
        HttpRequest.scheme = 'https'

    request.META['SERVER_NAME'] = server_url
    request.META['SERVER_PORT'] = server_port
    request.user = AnonymousUser()
    return request


def round(value):
    value = decimal.Decimal(value)
    return value.quantize(decimal.Decimal('1.'), rounding=decimal.ROUND_UP)


def human_format(number):
    if number == 0:
        return "0"
    format_string = '%.1f %s'
    units = ['', 'K', 'M', 'G', 'T', 'P']
    k = 1000.0
    magnitude = int(floor(log(number, k)))
    result = number / k ** magnitude
    if magnitude == 0:
        format_string = '%.0f %s'
    return format_string % (result, units[magnitude])


def field_with_prefix(field, prefix=''):
    if prefix:
        field = prefix + '__' + field
    return field


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-08-13 21:29


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0003_user_verify_mobile'),
    ]

    operations = [
        migrations.AddField(
            model_name='devicelogin',
            name='device_type',
            field=models.CharField(choices=[('a', 'Android'), ('i', 'iOS')], default='a', max_length=1, verbose_name='Device Type'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='devicelogin',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='Is Active'),
        ),
    ]



from django.db.models import Sum, Count, Q, F
from django.db.models.functions import Coalesce

from aparnik.contrib.suit.api.views import AdminListAPIView
from aparnik.contrib.suit.api.views import ModelAdminSortAPIView
from .serializers import Audit, AuditAdminListSerializer


class AuditSortAdminAPIView(ModelAdminSortAPIView):

    """
        A view that returns the count of active users in JSON.
        """
    def __init__(self, *args, **kwargs):
        super(AuditSortAdminAPIView, self).__init__(*args, **kwargs)

        command_dict = {

        }
        # this name of
        command_dict.update(Audit.sort_audit(type=Audit.ACTION_LOGIN))
        command_dict.update(Audit.sort_audit(type=Audit.ACTION_LOGOUT))
        command_dict.update(Audit.sort_audit(type=Audit.ACTION_LOGIN_FAILED))
        self.command_dict.update(command_dict)


class AuditAdminListAPIView(AdminListAPIView):
    serializer_class = AuditAdminListSerializer
    queryset = Audit.objects.all()

    def get_sort_api(self, request):
        return AuditSortAdminAPIView(request=self.request)



# -*- coding: utf-8 -*-


from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.contrib.province.models import City

User = get_user_model()


# Create your models here.

class BaseEducationManager(BaseModelManager):
    def get_queryset(self):
        return super(BaseEducationManager, self).get_queryset()

    def get_this_education(self, user):
        return self.get_queryset().all()

    def active(self):
        return self.get_queryset().filter(is_published=True)

    def model_educations(self, model_obj, user_obj=None):
        objs = self.active().filter(model_obj=model_obj)
        if user_obj:
            objs = objs | self.get_queryset().filter(user_obj=user_obj)
        return objs


class BaseEducation(BaseModel):
    is_published = models.BooleanField(default=False, verbose_name=_('Published'))

    objects = BaseEducationManager()

    def __init__(self, *args, **kwargs):
        super(BaseEducation, self).__init__(*args, **kwargs)

    class Meta:
        verbose_name = _('Base Education')
        verbose_name_plural = _('Base Education')

    # def get_education_url(self):
    #     return reverse('api:educations:detail', kwargs={'education_id':self.id})
    #
    # def get_institute_url(self):
    #     return reverse('api:educations:list', kwargs={'institute_id': self.id})


class DegreeManager(BaseEducationManager):
    def get_queryset(self):
        return super(DegreeManager, self).get_queryset()

    def active(self):
        return self.get_queryset().filter(is_published=True)


class Degree(BaseEducation):
    name = models.CharField(max_length=50, verbose_name=_('Title'))
    objects = DegreeManager()

    def __init__(self, *args, **kwargs):
        super(Degree, self).__init__(*args, **kwargs)

    def __str__(self):
        return str(self.name)

    class Meta:
        verbose_name = _('Degree')
        verbose_name_plural = _('Degrees')


class FieldSubjectManager(BaseEducationManager):
    def get_queryset(self):
        return super(FieldSubjectManager, self).get_queryset()


class FieldSubject(BaseEducation):
    name = models.CharField(max_length=50, verbose_name=_('Title'))
    parent_obj = models.ForeignKey('self', null=True, blank=True, on_delete=models.CASCADE, verbose_name=_('Parent Field Subject'))

    objects = FieldSubjectManager()

    def __str__(self):
        return str(self.name)

    class Meta:
        verbose_name = _('Field Subject')
        verbose_name_plural = _('Field Subjects')


class InstituteManager(BaseEducationManager):
    def get_queryset(self):
        return super(InstituteManager, self).get_queryset()


class Institute(BaseEducation):
    name = models.CharField(max_length=40, verbose_name=_('Institute Name'))
    cities = models.ManyToManyField(City, through='CityInstitute')
    objects = InstituteManager()

    class Meta:
        verbose_name = _('Institute')
        verbose_name_plural = _('Institutes')

    def __str__(self):
        return str(self.name)


class CityInstituteManager(BaseEducationManager):
    def get_queryset(self):
        return super(CityInstituteManager, self).get_queryset()


class CityInstitute(BaseEducation):
    city = models.ForeignKey(City, on_delete=models.CASCADE, verbose_name=_('City'))
    Institute = models.ForeignKey(Institute, null=True, on_delete=models.CASCADE, verbose_name=_('Institude'))

    objects = CityInstituteManager()

    # def get_queryset(self):
    #     return super(BaseEducationManager, self).get_queryset()

    class Meta:
        verbose_name = _('CityInstitute')
        verbose_name_plural = _('CityInstitutes')


class EducationManager(BaseEducationManager):
    def get_queryset(self):
        return super(EducationManager, self).get_queryset()


class Education(BaseEducation):
    user_obj = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('User'))
    degree_obj = models.ForeignKey(Degree, null=True, on_delete=models.CASCADE, verbose_name=_('Degree'))
    institute_obj = models.ForeignKey(Institute, null=True, on_delete=models.CASCADE, verbose_name=_('Institute'))
    field_subject_obj = models.ForeignKey(FieldSubject, null=True, on_delete=models.CASCADE, verbose_name=_('Field Subject'))
    receive_date = models.DateTimeField(null=True, verbose_name=_('Receive Date'))
    average = models.FloatField(null=True, max_length=3, verbose_name=_('Average'))
    description = models.TextField(max_length=200, null=True, blank=True, verbose_name=_('Descritpion'))

    objects = EducationManager()

    def __init__(self, *args, **kwargs):
        super(Education, self).__init__(*args, **kwargs)

    class Meta:
        verbose_name = _('Education')
        verbose_name_plural = _('Educations')

    def __str__(self):
        return str(self.user_obj)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 11:10


from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('basemodels', '0001_initial'),
        ('products', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Membership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_read', models.BooleanField(default=False, verbose_name='Is Read')),
            ],
            options={
                'verbose_name': 'Membership',
                'verbose_name_plural': 'Memberships',
            },
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('title', models.CharField(max_length=50, verbose_name='Title')),
                ('description', models.CharField(max_length=255, verbose_name='Message')),
                ('type', models.CharField(choices=[('i', '\\u0627\\u0637\\u0644\\u0627\\u0639\\u0627\\u062a'), ('s', '\\u0645\\u0648\\u0641\\u0642\\u06cc\\u062a'), ('w', '\\u0647\\u0634\\u062f\\u0627\\u0631'), ('e', '\\u062e\\u0637\\u0627')], max_length=1, verbose_name='Type')),
                ('is_for_all_users', models.BooleanField(default=True, verbose_name='Is For All Users')),
                ('datetime', models.DateTimeField(auto_now_add=True, verbose_name='Datetime')),
                ('from_user_obj', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notificaiton_from_user', to=settings.AUTH_USER_MODEL, verbose_name='\\u0627\\u0632 \\u06a9\\u0627\\u0631\\u0628\\u0631')),
                ('model_obj', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notification_model_obj', to='basemodels.BaseModel', verbose_name='Model')),
                ('products_buy', models.ManyToManyField(blank=True, related_name='notification_products_buy', to='products.Product', verbose_name='Purchase specific products')),
                ('products_buy_did_not', models.ManyToManyField(blank=True, related_name='notification_products_buy_did_not', to='products.Product', verbose_name='Did not purchase specific products ')),
                ('users', models.ManyToManyField(through='notifications.Membership', to=settings.AUTH_USER_MODEL, verbose_name='Membership')),
            ],
            options={
                'verbose_name': 'Notification',
                'verbose_name_plural': 'Notifications',
            },
            bases=('basemodels.basemodel',),
        ),
        migrations.AddField(
            model_name='membership',
            name='notification',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='notifications.Notification', verbose_name='Notification'),
        ),
        migrations.AddField(
            model_name='membership',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.CreateModel(
            name='NotificationForSingleUser',
            fields=[
            ],
            options={
                'verbose_name': 'Notification For Single User',
                'proxy': True,
                'verbose_name_plural': 'Notifications For Single User',
                'indexes': [],
            },
            bases=('notifications.notification',),
        ),
    ]



from django.conf.urls import url, include

from .views import CouponSortAdminAPIView, CouponAdminListAPIView

app_name = 'coupons'

urlpatterns = [
    url(r'^$', CouponAdminListAPIView.as_view(), name='list'),
    url(r'^sort/$', CouponSortAdminAPIView.as_view(), name='sort'),
]



from django.conf.urls import url

from .views import BaseEducationListAPIView, EducationCreateAPIView, \
    InstituteListAPIView, FieldSubjectListAPIView, EducationUpdateAPIView, DegreeListAPIView

app_name = 'educations'

urlpatterns = [
    url(r'^$', BaseEducationListAPIView.as_view(), name='list'),
    url(r'^create/$', EducationCreateAPIView.as_view(), name='create'),
    url(r'^(?P<education_id>[\w-]+)/update', EducationUpdateAPIView.as_view(), name='update'),
    url(r'^institutes/$', InstituteListAPIView.as_view(), name='institude-list'),
    url(r'^field-subjects/$', FieldSubjectListAPIView.as_view(), name='field-subject-list'),
    url(r'^degree/$', DegreeListAPIView.as_view(), name='degree-list'),
]



from django.conf.urls import url, include

from .views import CourseSortAdminAPIView, CourseAdminListAPIView

app_name = 'courses'

urlpatterns = [
    url(r'^$', CourseAdminListAPIView.as_view(), name='list'),
    url(r'^sort/$', CourseSortAdminAPIView.as_view(), name='sort'),
]



from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import ( AllowAny, IsAuthenticated )

from .serializers import ContactUsDestailSerializer
from ..models import ContactUs


class ContactUsListAPIView(ListAPIView):
    serializer_class = ContactUsDestailSerializer
    permission_classes = [IsAuthenticated]
    queryset = ContactUs.objects.active()


class ContactUsDetailAPIView(RetrieveAPIView):
    serializer_class = ContactUsDestailSerializer
    permission_classes = [AllowAny]
    lookup_field = 'id'
    lookup_url_kwarg = 'id'

    def get_object(self):
        return ContactUs.objects.active()


class ContactUsCreateAPIView(CreateAPIView):
    serializer_class = ContactUsDestailSerializer
    permission_classes = [AllowAny]
    queryset = ContactUs.objects.active()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-09 23:20


import ckeditor_uploader.fields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('filefields', '0004_filefield_title'),
        ('basemodels', '0004_auto_20181103_2233'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='News',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('title', models.CharField(max_length=200, verbose_name='\\u0639\\u0646\\u0648\\u0627\\u0646')),
                ('slug', models.SlugField(allow_unicode=True, max_length=300, unique=True, verbose_name='Slug')),
                ('content', ckeditor_uploader.fields.RichTextUploadingField(verbose_name='\\u0645\\u062d\\u062a\\u0648\\u0627')),
                ('is_published', models.BooleanField(default=True, verbose_name='Is Published')),
                ('publish_date', models.DateTimeField(default=django.utils.timezone.now, null=True, verbose_name='\\u062a\\u0627\\u0631\\u06cc\\u062e \\u0627\\u0646\\u062a\\u0634\\u0627\\u0631')),
                ('cover_images', models.ManyToManyField(to='filefields.FileField', verbose_name='Cover Images')),
                ('user_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='\\u06a9\\u0627\\u0631\\u0628\\u0631')),
            ],
            options={
                'verbose_name': 'News',
                'manager_inheritance_from_future': True,
                'verbose_name_plural': 'NEWS',
            },
            bases=('basemodels.basemodel',),
        ),
    ]



from django.utils.translation import ugettext_lazy as _
from django.shortcuts import get_object_or_404
from django.contrib.auth import get_user_model
from rest_framework.generics import RetrieveAPIView, ListAPIView
from rest_framework.permissions import (IsAuthenticated)
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.views import APIView
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError
from aparnik.contrib.users.models import User
from .serializers import CoSaleUserListSerializer, CoSaleListSerializer
from ..models import CoSale


class CoSaleUserListAPIView(ListAPIView):
    serializer_class = CoSaleUserListSerializer
    permission_classes = [IsAuthenticated]

    # filter_backends = (filters.SearchFilter,)

    def get_queryset(self):
        user = self.request.user
        return [user]


class CoSaleListAPIView(ListAPIView):
    serializer_class = CoSaleListSerializer
    permission_classes = [IsAuthenticated]

    # filter_backends = (filters.SearchFilter,)

    def get_queryset(self):
        user = self.request.user
        return CoSale.objects.this_user(user)


# class OrderDetailAPIView(RetrieveAPIView):
#     serializer_class = OrderDetailSerializer
#     permission_classes = [IsAuthenticated]
#     lookup_url_kwarg = 'id'
#     lookup_field = 'id'
#
#     def get_queryset(self):
#         user = self.request.user
#
#         return Order.objects.get_this_user(user=user)
#
#
# class AddProductOrderAPIView(APIView):
#     permission_classes = [IsAuthenticated]
#
#     def post(self, request, id=None, format=None, *args, **kwargs):
#         status = HTTP_400_BAD_REQUEST
#         user = self.request.user
#         # id = product_id
#         content = {
#             'success': False,
#             'msg': '',
#             'data': {
#             },
#         }
#         order = None
#         products = None
#         products_id = request.POST.get('products_id').split(',')
#         try:
#             products = Product.objects.filter(id__in=products_id)
#             if id:
#                 order = get_object_or_404(Order.objects.all(), id=id)
#             else:
#                 order = Order.objects.create(user=user)
#         except Exception as e:
#             content["msg"] = e
#             content["success"] = False
#             content["data"] = {}
#             return Response(content, status=status)
#
#         for product in products.all():
#             order.add_item(product=product)
#
#         order.save()
#
#         content['data']['order'] = OrderSummarySerializer(order, many=False, read_only=True,
#                                                           context={'request': self.request}).data
#         content['success'] = True
#         status = HTTP_200_OK
#         return Response(content, status=status)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-14 12:30


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0004_filefield_title'),
        ('products', '0006_auto_20181113_1848'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProductProperty',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100, verbose_name='Title')),
                ('icon', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='filefields.FileField', verbose_name='Icon')),
            ],
            options={
                'verbose_name': 'Product Property',
                'verbose_name_plural': 'Product Properties',
            },
        ),
        migrations.CreateModel(
            name='ProductPropertyMembership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=255, verbose_name='Content')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='products.Product', verbose_name='Product')),
                ('property', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='products.ProductProperty', verbose_name='Property')),
            ],
            options={
                'verbose_name': 'Product Property Membership',
                'verbose_name_plural': 'Product Property Membership',
            },
        ),
        migrations.AddField(
            model_name='product',
            name='properties',
            field=models.ManyToManyField(through='products.ProductPropertyMembership', to='products.ProductProperty', verbose_name='Product Properties'),
        ),
    ]



from django.conf.urls import url, include

from .views import CourseListAPIView, CourseDetailAPIView, CourseUserListAPIView, CourseCreateAPIView

app_name = 'courses'

urlpatterns = [
    url(r'^$', CourseListAPIView.as_view(), name='list'),
    url(r'^admin/', include('aparnik.packages.educations.courses.api.admin.urls', namespace='admin')),
    url(r'^user/$', CourseUserListAPIView.as_view(), name='user'),
    url(r'^(?P<course_id>\d+)/$', CourseListAPIView.as_view(), name='detail'),
    # url(r'^(?P<course_id>\d+)/step/', include('courses.api.urls-steps', namespace='step')),
    # url(r'^(?P<course_id>\d+)/step/', include('courses.api.urls-steps', namespace='step')),
    url(r'^(?P<course_id>\d+)/file/', include('aparnik.packages.educations.courses.api.urls-files', namespace='files')),
    url(r'^create/$', CourseCreateAPIView.as_view(), name='create'),

]


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-04-17 12:26


import aparnik.contrib.users.models
import aparnik.utils.fields
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        ('province', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('sys_id', models.AutoField(primary_key=True, serialize=False, verbose_name='System ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, verbose_name='UUID')),
                ('username', aparnik.utils.fields.PhoneField(max_length=11, unique=True, validators=[django.core.validators.RegexValidator(code=b'nomatch', message='phone is not valid, please insert with code', regex=b'^0(?!0)\\d{2}([0-9]{8})$')], verbose_name='Mobile')),
                ('email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='Email')),
                ('first_name', models.CharField(blank=True, max_length=12, null=True, verbose_name='Name')),
                ('last_name', models.CharField(blank=True, max_length=30, null=True, verbose_name='Family')),
                ('sex', models.CharField(choices=[('M', '\\u0645\\u0631\\u062f'), ('F', '\\u0632\\u0646')], default='M', max_length=1, verbose_name='Sex')),
                ('avatar', models.ImageField(blank=True, null=True, upload_to='file', verbose_name='Avatar')),
                ('birthdate', models.DateTimeField(blank=True, null=True, verbose_name='Birthdate')),
                ('is_staff', models.BooleanField(default=False, verbose_name='Is Staff')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('token', models.CharField(blank=True, max_length=4, null=True, verbose_name='Token')),
                ('token_time', models.DateTimeField(blank=True, null=True, verbose_name='Token Time')),
                ('limit_device_login', models.PositiveIntegerField(default=0, help_text='If the number is set to 0, then the value in the settings will apply', verbose_name='Limit device login')),
                ('invitation_code', models.CharField(blank=True, editable=False, max_length=36, null=True, verbose_name='Invitation Code')),
                ('current_city', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='province.City', verbose_name='Current City')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'User',
                'verbose_name_plural': 'Users',
            },
        ),
        migrations.CreateModel(
            name='DeviceLogin',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('version_number', models.FloatField(verbose_name='Version Number')),
                ('device_id', models.CharField(max_length=255, verbose_name='Device Id')),
                ('os_version', models.CharField(max_length=255, verbose_name='OS Version')),
                ('device_model', models.CharField(max_length=255, verbose_name='Device Model')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('update_at', models.DateTimeField(auto_now=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='aparnik_users.User', verbose_name='User')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-28 21:16


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0003_remove_product_slider_obj'),
        ('sliders', '0002_slidersegment'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='slider_segment_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='products_sliders', to='sliders.SliderSegment', verbose_name='Slider Segment'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin


from aparnik.contrib.users.admin import get_update_at, get_user_search_fields
from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.packages.shops.products.admin import ProductAdmin

from .models import Subscription, SubscriptionOrder


# Register your models here.
class SubscriptionAdmin(ProductAdmin):
    fields = ['duration', 'description', 'type', 'products']
    list_display = ['duration']
    search_fields = ['duration']
    list_filter = ['duration']
    exclude = []
    raw_id_fields = []
    dynamic_raw_id_fields = ['products']

    def __init__(self, *args, **kwargs):
        Klass = SubscriptionAdmin
        Klass_parent = ProductAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields


class SubscriptionOrderAdmin(BaseModelAdmin):
    fields = ['subscription_obj', 'order_obj', 'expire_at']
    list_display = ['subscription_obj', 'order_obj', 'expire_at', get_update_at]
    search_fields = get_user_search_fields('order_obj__user') + ['subscription_obj', 'order_obj', ]
    list_filter = []
    exclude = []
    raw_id_fields = ['subscription_obj', 'order_obj']
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = SubscriptionOrderAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields


admin.site.register(Subscription, SubscriptionAdmin)
admin.site.register(SubscriptionOrder, SubscriptionOrderAdmin)




from django.db import models
from django.db.models import Q, Count, Sum
from django.db.models.functions import Coalesce
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.signals import user_logged_in, user_logged_out, user_login_failed
from django.contrib.auth import get_user_model
from django.dispatch import receiver


from aparnik.utils.utils import field_with_prefix
User = get_user_model()


class AuditManager(models.Manager):
    def get_queryset(self):
        return super(AuditManager, self).get_queryset()


class Audit(models.Model):

    ACTION_LOGIN = 'li'
    ACTION_LOGOUT = 'lo'
    ACTION_LOGIN_FAILED = 'lf'
    ACTION_CHOICES = (
        (ACTION_LOGIN, _('Action login')),
        (ACTION_LOGOUT, _('Action logout')),
        (ACTION_LOGIN_FAILED, _('Action login failed')),
    )

    action = models.CharField(max_length=64, choices=ACTION_CHOICES, verbose_name=_('Action'))
    ip = models.GenericIPAddressField(null=True, verbose_name=_('IP'))
    user_obj = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE, verbose_name=_('User'))
    extra = models.TextField(null=True, blank=True, verbose_name=_('Extra data'))

    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = AuditManager()

    def __str__(self):
        return '{0} - {1}'.format(self.action, self.ip)

    @staticmethod
    def sort_audit(return_key='audit', prefix='', type=ACTION_LOGIN):
        display = None
        for x in Audit.ACTION_CHOICES:
            if x[0] == type:
                display = x[1]
                break
        if prefix != '':
            field = 'id'
        else:
            field = 'ip'

        sort = {
            return_key + type: {
                'label': display,
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count(field_with_prefix(field=field, prefix=prefix), filter=Q(**{field_with_prefix(prefix=prefix, field='action'): type}))
                },
                'key_sort': 'sort_count',
            }
        }
        return sort


@receiver(user_logged_in)
def user_logged_in_callback(sender, request, user, **kwargs):
    ip = request.META.get('REMOTE_ADDR')
    Audit.objects.create(action=Audit.ACTION_LOGIN, ip=ip, user_obj=user)


@receiver(user_logged_out)
def user_logged_out_callback(sender, request, user, **kwargs):
    ip = request.META.get('REMOTE_ADDR')
    Audit.objects.create(action=Audit.ACTION_LOGOUT, ip=ip, user_obj=user)


@receiver(user_login_failed)
def user_login_failed_callback(sender, credentials, request, **kwargs):
    username = credentials.get('username', None)
    ip = request.META.get('REMOTE_ADDR')
    user = None
    if username:
        try:
            user = User.objects.get(username=username)
            # در این حالت فقط زمانی که یوزر نیم پیدا نشه لاگ یوزر نیم ثبت میشه و از افزونگی داده جلوگیری می کنه
            username = None
        except:
            pass
    Audit.objects.create(action=Audit.ACTION_LOGIN_FAILED, ip=ip, user_obj=user, extra=username)


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-06-08 19:18


import aparnik.contrib.tickets.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_enumfield.db.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('filefields', '0010_filefield_is_decrypt_needed'),
        ('basemodels', '0010_basemodel_visit_count'),
    ]

    operations = [
        migrations.CreateModel(
            name='Ticket',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('priority', django_enumfield.db.fields.EnumField(default=0, enum=aparnik.contrib.tickets.models.TicketPriority, verbose_name='Priority')),
                ('status', django_enumfield.db.fields.EnumField(default=0, enum=aparnik.contrib.tickets.models.TicketStatus, verbose_name='Status')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, verbose_name='UUID')),
                ('department', django_enumfield.db.fields.EnumField(default=0, enum=aparnik.contrib.tickets.models.TicketDepartment, verbose_name='Department')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('user_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Ticket',
                'manager_inheritance_from_future': True,
                'verbose_name_plural': 'Tickets',
            },
            bases=('basemodels.basemodel',),
        ),
        migrations.CreateModel(
            name='TicketConversation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.TextField(verbose_name='Content')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
                ('files_obj', models.ManyToManyField(to='filefields.FileField', verbose_name='Files')),
                ('ticket_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='tickets.Ticket', verbose_name='Ticket')),
                ('user_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'ordering': ['-created_at'],
                'verbose_name': 'Ticket Conversation',
                'verbose_name_plural': 'Ticket Conversations',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-21 09:50


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('managements', '0004_fieldlist_group'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fieldlist',
            name='group',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.Group', verbose_name='group'),
        ),
    ]



from django.conf.urls import url, include

from .views import ProductSharingListAPIView, ProductSharingSetAPIView

app_name = 'productsharing'

urlpatterns = [
    url(r'^$', ProductSharingListAPIView.as_view(), name='list'),
    url(r'^(?P<product_id>\d+)/users/$', ProductSharingListAPIView.as_view(), name='product-user-share-list'),
    url(r'^(?P<product_id>\d+)/set/$', ProductSharingSetAPIView.as_view(), name='product-share-set'),
    # url(r'^model/(?P<model_id>\d+)/set/$', BookmarkSetAPIView.as_view(), name='set'),

]


from django.db.models import Sum, Count, Q, F
from django.db.models.functions import Coalesce


from aparnik.contrib.audits.models import Audit
from aparnik.packages.shops.vouchers.models import Voucher
from aparnik.packages.shops.coupons.models import Coupon
from aparnik.packages.shops.orders.models import Order
from aparnik.contrib.suit.api.views import AdminListAPIView
from aparnik.contrib.suit.api.views import ModelAdminSortAPIView
from aparnik.packages.educations.progresses.models import ProgressSummary
from .serializers import User, UserAdminListSerializer


class UserSortAdminAPIView(ModelAdminSortAPIView):

    """
        A view that returns the count of active users in JSON.
        """
    def __init__(self, *args, **kwargs):
        super(UserSortAdminAPIView, self).__init__(*args, **kwargs)

        command_dict = {
            # TODO: read bellow line
            # این گزارش باید بر اساس تاریخ به مدیر گفته شود
            'register_date': {
                'label': 'ثبت نام',
                'queryset_filter': Q(),
                'annotate_command': {'sort_count': F('created_at')},
                'key_sort': 'sort_count',
            },
            'qa': {
                'label': 'پرسش و پاسخ',
                'queryset_filter': Q(),
                'annotate_command': {'sort_count': Count('qa')},
                'key_sort': 'sort_count',
            },
            'reviews': {
                'label': 'نظر',
                'queryset_filter': Q(),
                'annotate_command': {'sort_count': Count('review')},
                'key_sort': 'sort_count',
            },

        }
        # this name of
        command_dict.update(Audit.sort_audit(prefix='audit', type=Audit.ACTION_LOGIN))
        command_dict.update(Audit.sort_audit(prefix='audit', type=Audit.ACTION_LOGOUT))
        command_dict.update(Audit.sort_audit(prefix='audit', type=Audit.ACTION_LOGIN_FAILED))
        command_dict.update(Voucher.sort_voucher(prefix='order__items__voucher_item_spent', return_key='users_voucher'))
        command_dict.update(Coupon.sort_redeem(prefix='coupons', return_key='users_redeem'))
        command_dict.update(ProgressSummary.sort_progress(prefix='progress_user'))
        command_dict.update(Order.sort_buy(prefix='order'))
        command_dict.update(Order.sort_buy_waiting(prefix='order'))
        command_dict.update(Order.sort_buy_wallet(prefix='order'))
        command_dict.update(Order.sort_buy_bank(prefix='order'))

        self.command_dict.update(command_dict)


class UserAdminListAPIView(AdminListAPIView):
    serializer_class = UserAdminListSerializer
    queryset = User.objects.all()

    def get_sort_api(self, request):
        return UserSortAdminAPIView(request=self.request)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 12:06


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('payments', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bank',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.IntegerField(default=-2, verbose_name='Status')),
                ('authority_id', models.CharField(max_length=255, verbose_name='Authority ID')),
                ('ref_id', models.CharField(blank=True, max_length=255, null=True, verbose_name='Bank Reference ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update At')),
                ('payment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='payments.Payment', verbose_name='Payment')),
            ],
            options={
                'ordering': ['created_at'],
                'verbose_name': 'Bank',
                'verbose_name_plural': 'Banks',
            },
        ),
    ]



from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import ( AllowAny, IsAuthenticated )
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.views import APIView

from .serializers import AboutusDestailSerializer
from ..models import Information


class AboutusDetailAPIView(RetrieveAPIView):
    serializer_class = AboutusDestailSerializer
    permission_classes = [AllowAny]

    def get_object(self):
        return Information.objects.get_active().last()



from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import ( AllowAny, IsAuthenticated )

from .serializers import FAQDetailsSerializer
from ..models import FAQ


class FAQDetailAPIView(RetrieveAPIView):
    serializer_class = FAQDetailsSerializer
    permission_classes = [AllowAny]

    def get_object(self):
        return FAQ.objects.active().last()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-17 16:50


import aparnik.contrib.notifications.models
from django.db import migrations
import django_enumfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='notification_send_type',
            field=django_enumfield.db.fields.EnumField(blank=True, default=0, enum=aparnik.contrib.notifications.models.NotificationType, null=True, verbose_name='Notification send type'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-14 15:37


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0002_page_english_title'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='is_show_in_home',
            field=models.BooleanField(default=True, verbose_name='Is Show in home'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin, messages
from django.http import HttpResponseRedirect
from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.contrib.managements.models import Management, FieldList, ManagementActions, ManagementRoles, \
    ManagementPermission
from .forms import ManagementForm, FieldListForm


# Register your models here.
class ManagementRolesAdmin(BaseModelAdmin):
    fields = ['title', 'management_obj']
    list_display = ['title']
    list_filter = []
    search_fields = []
    exclude = []
    save_as = True

    def __init__(self, *args, **kwargs):
        Klass = ManagementRolesAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude

    class Meta:
        model = ManagementRoles


class ManagementAdmin(BaseModelAdmin):
    change_list_template = "update-management.html"
    fields = ['group', 'application', 'fields', 'start_date', 'end_date']
    list_display = ['group', 'application', 'start_date', 'end_date']
    list_filter = ['group']
    search_fields = ['group__name', 'fields__id', 'application']
    raw_id_fields = []
    dynamic_raw_id_fields = ['group', 'fields']
    exclude = []
    save_as = True
    form = ManagementForm

    def __init__(self, *args, **kwargs):
        Klass = ManagementAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude

    def get_urls(self):
        urls = super(ManagementAdmin, self).get_urls()
        my_urls = [
            url(r'^update-management/$', self.admin_site.admin_view(self.update_management))
        ]
        return my_urls + urls

    def update_management(self, request):
        group_id = int(request.POST['group_id'])
        try:
            FieldList.objects.update_fields(group_id)
            Management.objects.update_apps(group_id)
        except:
            self.message_user(request, _("Aparnik management group id does not exist ") + str(group_id),
                              level=messages.ERROR)
            return HttpResponseRedirect("../")

        self.message_user(request, _("Aparnik management data updated successfully for group id ") + str(group_id))
        return HttpResponseRedirect("../")

    def save_model(self, request, obj, form, change):
        obj.is_superuser = request.user.is_superuser
        super(ManagementAdmin, self).save_model(request, obj, form, change)

    class Meta:
        model = Management


class FieldListAdmin(BaseModelAdmin):
    fields = ['model', 'name', 'group', 'actions', 'permission', 'is_enable', 'is_sharable', 'default']
    list_display = ['name', 'model', 'group', 'default', 'is_enable']
    list_filter = ['is_enable', 'group', 'model', ]
    search_fields = ['name', 'model']
    exclude = []
    save_as = True
    form = FieldListForm
    actions = ['make_enable', 'make_disable']

    def make_enable(self, request, queryset):
        rows_updated = queryset.update(is_enable=True)
        if rows_updated == 1:
            message_bit = "1 field was"
        else:
            message_bit = "%s fields were" % rows_updated
        self.message_user(request, _("%s successfully marked as enabled.") % message_bit)

    make_enable.short_description = _("Mark selected fields as enabled")

    def make_disable(self, request, queryset):
        rows_updated = queryset.update(is_enable=False)

        if rows_updated == 1:
            message_bit = "1 field was"
        else:
            message_bit = "%s fields were" % rows_updated

        self.message_user(request, _("%s successfully marked as disabled.") % message_bit)

    make_disable.short_description = _("Mark selected fields as disabled")

    def __init__(self, *args, **kwargs):
        Klass = FieldListAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude

    class Meta:
        model = FieldList


class ManagementActionsAdmin(BaseModelAdmin):
    fields = ['title', 'description']
    list_display = ['title', 'description']
    list_filter = []
    search_fields = []
    exclude = []
    save_as = True

    def __init__(self, *args, **kwargs):
        Klass = ManagementActionsAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude

    def save_model(self, request, obj, form, change):
        obj.is_superuser = request.user.is_superuser
        super(ManagementActionsAdmin, self).save_model(request, obj, form, change)

    class Meta:
        model = ManagementActions


class ManagementPermissionAdmin(BaseModelAdmin):
    fields = ['title', 'description']
    list_display = ['title', 'description']
    list_filter = []
    search_fields = []
    exclude = []
    save_as = True

    def __init__(self, *args, **kwargs):
        Klass = ManagementPermissionAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude

    class Meta:
        model = ManagementPermission


# Change to users admin class
admin.site.register(Management, ManagementAdmin)
admin.site.register(ManagementPermission, ManagementPermissionAdmin)
admin.site.register(FieldList, FieldListAdmin)
admin.site.register(ManagementRoles, ManagementRolesAdmin)
admin.site.register(ManagementActions, ManagementActionsAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-02 15:43


from django.db import migrations


def negative_default_value(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Object = apps.get_model('products', 'Product')
    Object.objects.filter(aparnik_bon_return_value=0).update(aparnik_bon_return_value=-1)
    Object.objects.filter(aparnik_bon_return_expire_value=0).update(aparnik_bon_return_expire_value=-1)
    Object.objects.filter(maximum_use_aparnik_bon_value=0).update(maximum_use_aparnik_bon_value=-2)
    Object.objects.filter(has_permit_use_wallet_value=0).update(has_permit_use_wallet_value=-1)


def zero_default_value(apps, schema_editor):
    Object = apps.get_model('products', 'Product')
    Object.objects.filter(aparnik_bon_return_value=-1).update(aparnik_bon_return_value=0)
    Object.objects.filter(aparnik_bon_return_expire_value=-1).update(aparnik_bon_return_expire_value=0)
    Object.objects.filter(maximum_use_aparnik_bon_value=-2).update(maximum_use_aparnik_bon_value=0)
    Object.objects.filter(has_permit_use_wallet_value=-1).update(has_permit_use_wallet_value=0)


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0010_auto_20181202_1542'),
    ]

    operations = [
        migrations.RunPython(negative_default_value, reverse_code=zero_default_value),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-17 20:59


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0003_order_address_obj'),
    ]

    operations = [
        migrations.AddField(
            model_name='orderitem',
            name='is_tax',
            field=models.BooleanField(default=False, verbose_name='Is Tax'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.filefields.models import FileField
from aparnik.contrib.segments.models import BaseSegment, BaseSegmentManager
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager


# Create your models here.
class SocialNetworkManager(BaseModelManager):
    def get_queryset(self):
        return super(SocialNetworkManager, self).get_queryset()


class SocialNetwork(BaseModel):
    link = models.URLField(
        max_length=255,
        verbose_name=_('Link')
    )

    icon = models.ForeignKey(
        FileField,
        on_delete=models.CASCADE,
        verbose_name=_('Icon'),
    )

    title = models.CharField(
        verbose_name=_('Title'),
        max_length=31,
        default=None
    )
    value = models.CharField(max_length=255, null=True, blank=True, verbose_name=_('Value'))

    android_app_shortcut = models.CharField(max_length=200, null=True, blank=True, verbose_name=_('Android shortcut'))
    ios_app_shortcut = models.CharField(max_length=200, null=True, blank=True, verbose_name=_('iOS shortcut'))

    def __str__(self):
        return '%s, %s' % (self.title, self.link)

    def __init__(self, *args, **kwargs):
        super(SocialNetwork, self).__init__(*args, **kwargs)

    class Meta:
        verbose_name = _('Social Network')
        verbose_name_plural = _('Social Networks')


class SocialNetworkSegment(BaseSegment):

    objects = BaseSegmentManager()

    def __str__(self):
        return super(SocialNetworkSegment, self).__unicode__()

    class Meta:
        verbose_name = _('Social Segment')
        verbose_name_plural = _('Social Segments')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 12:06


from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('basemodels', '0001_initial'),
        ('orders', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Payment',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, verbose_name='Reference ID')),
                ('bank_reference', models.CharField(blank=True, editable=False, max_length=50, null=True, verbose_name='Bank Reference ID')),
                ('method', models.CharField(choices=[('b', 'Bank'), ('w', 'Wallet')], editable=False, max_length=3, verbose_name='Method payment')),
                ('status', models.CharField(choices=[('c', 'Cancel'), ('cs', 'Cancel by system'), ('cs', 'Cancel by user after pay begin'), ('co', 'Complete'), ('w', 'Waiting')], editable=False, max_length=10, verbose_name='Status')),
                ('call_back_url', models.CharField(blank=True, editable=False, max_length=255, null=True, verbose_name='Call Back URL')),
                ('order_obj', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='payments', to='orders.Order', verbose_name='Order')),
                ('user', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'payment',
                'verbose_name_plural': 'Payments',
            },
            bases=('basemodels.basemodel',),
        ),
    ]



# -*- coding: utf-8 -*-


from django.core.management import BaseCommand
from django.contrib.auth import get_user_model
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta

# from fcm_django.models import FCMDevice
from aparnik.contrib.notifications.models import Notification
from aparnik.contrib.notifiesme.models import NotifyMe
from aparnik.contrib.basemodels.models import BaseModel
from aparnik.packages.educations.courses.models import Course, CourseFile
from aparnik.packages.shops.products.models import Product
import datetime

User = get_user_model()


class Command(BaseCommand):
    # Show this when the user types help
    help = "send notifies me"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        current_time = now()
        # TODO: get last_time
        last_time = now() + relativedelta(minutes=-4)

        notifies_request_id = NotifyMe.objects.active().values_list('model_obj__id', flat=True).distinct()

        ids = []
        # parent, file and etc.....
        course_queryset = Course.objects.children_deep(course_ids=list(notifies_request_id))
        courses_ids = course_queryset.values_list('id', flat=True)
        courses_new_ids = []
        if course_queryset.filter(publish_date__range=[last_time, current_time]).exists():
            courses_new_ids = course_queryset.filter(publish_date__range=[last_time, current_time]).values_list('id', flat=True)
            # new course published

        file_queryset = CourseFile.objects.active().filter(course_obj_id__in=courses_ids, publish_date__range=[last_time, current_time])
        files_new_ids = []
        if file_queryset.exists():
            files_new_ids = file_queryset.values_list('id', flat=True)
            # new file publish

        if len(files_new_ids) or len(courses_new_ids):
            self.reindex(notifies_request_id, courses_new_ids, files_new_ids)
        finished_time = now()

        print(('send notifies me %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))

    def reindex(self, notifies_request_id, courses_new_ids, files_new_ids):
        union_ids = list(courses_new_ids) + list(files_new_ids)
        for nid in union_ids:

            ids = []
            try:
                ids.append(CourseFile.objects.get(id=nid).course_obj.id)
            except:
                pass
            finally:
                ids.append(nid)

            ids = ids + list(Course.objects.parents_deep(course_id=ids[0]).values_list('id', flat=True).distinct())

            ids = list(set(ids))

            compare_ids = [x for x in ids if x not in union_ids]

            if not(len(compare_ids) == len(ids)):
                users_id = NotifyMe.objects.active().filter(model_obj_id__in=ids).order_by('user_obj').values_list('user_obj', flat=True).distinct()
                product = Product.objects.get(id=nid)
                try:
                    Notification.objects.send_notification(
                        type=Notification.NOTIFICATION_INFO,
                        users=User.objects.filter(pk__in=users_id),
                        title='فایل جدید',
                        description='%s در دسترس قرار گرفت.' %product.title,
                        from_user_obj=None,
                        model_obj=product
                    )
                except Exception as e:
                    print((e.message))



# -*- coding: utf-8 -*-


from django.contrib.auth import get_user_model
from rest_framework import serializers
import datetime
from aparnik.api.serializers import ModelSerializer
from aparnik.contrib.basemodels.api.serializers import BaseModelListSerializer, BaseModelDetailSerializer, ModelListPolymorphicSerializer
from aparnik.packages.shops.cosales.models import CoSale
from aparnik.contrib.users.api.serializers import UserSummaryListSerializer

User = get_user_model()


class CoSaleUserListSerializer(ModelSerializer):
    id = serializers.SerializerMethodField()
    transaction_today_count = serializers.SerializerMethodField()
    summary = serializers.SerializerMethodField()
    user_subset_count = serializers.SerializerMethodField()
    today_earnings = serializers.SerializerMethodField()
    today_earnings_string = serializers.SerializerMethodField()
    user = serializers.SerializerMethodField()
    earning_invited_by = serializers.SerializerMethodField()
    earning_invited_by_string = serializers.SerializerMethodField()
    order_complete_count = serializers.SerializerMethodField()
    url_ios_app = serializers.SerializerMethodField()
    url_android_app = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()

    class Meta:
        model = User
        fields = [
            'id',
            'transaction_today_count',
            'user_subset_count',
            'summary',
            'today_earnings',
            'today_earnings_string',
            'user',
            # درآمد کسی که دعوت کرده
            'earning_invited_by',
            'earning_invited_by_string',
            'order_complete_count',
            'url_ios_app',
            'url_android_app',
            'resourcetype',
        ]

        read_only_fields = [
            'id',
            'transaction_today_count',
            'summary',
            'user_subset_count',
            'today_earnings',
            'today_earnings_string',
            'user',
            # درآمد کسی که دعوت کرده
            'earning_invited_by',
            'earning_invited_by_string',
            'order_complete_count',
            'url_ios_app',
            'url_android_app',
            'resourcetype',
        ]

    def get_id(self, obj):
        return obj.sys_id

    def get_transaction_today_count(self, obj):
        return CoSale.objects.history(user=obj, is_user_bought=False).count()

    def get_summary(self, obj):
        return obj.co_sale_summary

    def get_today_earnings(self, obj):
        return CoSale.objects.transaction_price(user=obj, is_user_bought=False)

    def get_today_earnings_string(self, obj):
        return CoSale.objects.transaction_price_string(user=obj, is_user_bought=False)

    def get_user(self, obj):
        return UserSummaryListSerializer(obj, many=False, read_only=True, context=self.context).data

    def get_user_subset_count(self, obj):
        return '%s' % obj.invite_by.get_invited_by().count()

    def get_earning_invited_by(self, obj):
        return CoSale.objects.transaction_price(user=obj, is_user_bought=True, start_date=datetime.datetime(100, 12, 31, 23, 59, 59))

    def get_earning_invited_by_string(self, obj):
        return CoSale.objects.transaction_price_string(user=obj, is_user_bought=True, start_date=datetime.datetime(100, 12, 31, 23, 59, 59))

    def get_order_complete_count(self, obj):
        return CoSale.objects.history(user=obj, is_user_bought=True, start_date=datetime.datetime(100, 12, 31, 23, 59, 59)).count()

    def get_url_ios_app(self, obj):
        # TODO: fix link
        return self.context['request'].build_absolute_uri('ios')

    def get_url_android_app(self, obj):
        # TODO: fix link
        return self.context['request'].build_absolute_uri('android')

    def get_resourcetype(self, obj):
        return 'CoSaleUser'


class CoSaleListSerializer(BaseModelListSerializer):
    user_bought = serializers.SerializerMethodField()
    order = serializers.SerializerMethodField()

    class Meta:
        model = CoSale
        fields = BaseModelListSerializer.Meta.fields + [
            'price',
            'price_string',
            'user_bought',
            'order',
            'created_at',
        ]

        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + [
            'price',
            'price_string',
            'user_bought',
            'order',
            'created_at',
        ]

    def get_user_bought(self, obj):
        return UserSummaryListSerializer(obj.user_bought_obj, many=False, read_only=True, context=self.context).data

    def get_order(self, obj):
        return ModelListPolymorphicSerializer(obj.order_obj, many=False, read_only=True, context=self.context).data


class CoSaleDetailsSerializer(BaseModelDetailSerializer, CoSaleListSerializer):

    def __init__(self, *args, **kwargs):
        super(CoSaleDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = CoSale
        fields = CoSaleListSerializer.Meta.fields + BaseModelDetailSerializer.Meta.fields + [

        ]

        read_only_fields = CoSaleListSerializer.Meta.read_only_fields + BaseModelListSerializer.Meta.read_only_fields + [

        ]



from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from rest_framework.views import APIView

from rest_framework.permissions import (AllowAny, IsAuthenticated)

from ..models import Support
from .serializers import SupportDetailsSerializer  # , Serializer, UserCreateSerializer


class SupportListAPIView(ListAPIView):
    serializer_class = SupportDetailsSerializer
    permission_classes = [AllowAny]
    queryset = Support.objects.active()
    # filter_backends = (filters.SearchFilter,)search_fields = ('title')


class SupportDetailAPIView(RetrieveAPIView):
    serializer_class = SupportDetailsSerializer
    queryset = Support.objects.active()
    permission_classes = [AllowAny]
    lookup_url_kwarg = 'id'
    lookup_field = 'id'



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-01 17:10


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0006_auto_20190501_1142'),
    ]

    operations = [
        migrations.AddField(
            model_name='filefield',
            name='seconds',
            field=models.BigIntegerField(default=0, verbose_name='\\u0632\\u0645\\u0627\\u0646'),
        ),
    ]
